#include "Source_objects.h"
#include "Source_resources.h"
#include "built_in.h"

// Extern memory interface
// MMC/SD Connections
sbit Mmc_Chip_Select at LATD0_bit;
sbit Mmc_Chip_Select_Direction at TRISD0_bit;
// end of MMC/SD
// TFT Get Data globals
unsigned long currentSector = -1, res_file_size;
// end of TFT Get Data
// End of extern memory interface


// TFT module connections
char TFT_DataPort at PORTJ;
sbit TFT_RST at LATD3_bit;
sbit TFT_BLED at LATC2_bit;
sbit TFT_RS at LATE0_bit;
sbit TFT_CS at LATD2_bit;
sbit TFT_RD at LATH1_bit;
sbit TFT_WR at LATH2_bit;
char TFT_DataPort_Direction at TRISJ;
sbit TFT_RST_Direction at TRISD3_bit;
sbit TFT_BLED_Direction at TRISC2_bit;
sbit TFT_RS_Direction at TRISE0_bit;
sbit TFT_CS_Direction at TRISD2_bit;
sbit TFT_RD_Direction at TRISH1_bit;
sbit TFT_WR_Direction at TRISH2_bit;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at LATA0_bit;
sbit DriveX_Right at LATD7_bit;
sbit DriveY_Up at LATA1_bit;
sbit DriveY_Down at LATD4_bit;
sbit DriveX_Left_Direction at TRISA0_bit;
sbit DriveX_Right_Direction at TRISD7_bit;
sbit DriveY_Up_Direction at TRISA1_bit;
sbit DriveY_Down_Direction at TRISD4_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 800;
char PenDown;
typedef unsigned long TPointer;
TPointer PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TCLabel *local_clabel;
TCLabel *exec_clabel;
int clabel_order;
TImage *local_image;
TImage *exec_image;
int image_order;
TCBox *local_cbox;
TCBox *exec_cbox;
int cbox_order;
TProgressBar *local_progress_bar;
TProgressBar *exec_progress_bar;
int progress_bar_order;


void Init_ADC() {
  PLLEN_bit = 1;
  Delay_ms(150);
  ANCON0 = 0xF0; // All pins to digital
  ANCON1 = 0xFF;
  ADC_Init();
}

char* TFT_Get_Data(unsigned long offset, unsigned int count, unsigned int *num) {
unsigned long start_sector;
unsigned int pos;

  start_sector = Mmc_Get_File_Write_Sector() + offset/512;
  pos = (unsigned long)offset%512;

  if(start_sector == currentSector+1) {
    Mmc_Multi_Read_Sector(f16_sector.fSect);
    currentSector = start_sector;
  } else if (start_sector != currentSector) {
    if(currentSector != -1)
      Mmc_Multi_Read_Stop();
    Mmc_Multi_Read_Start(start_sector);
    Mmc_Multi_Read_Sector(f16_sector.fSect);
    currentSector = start_sector;
  }

  if(count>512-pos)
    *num = 512-pos;
  else
    *num = count;

  return f16_sector.fSect+pos;
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init(320, 240);
  TFT_Set_Ext_Buffer(TFT_Get_Data);

  TP_TFT_Init(320, 240, 0, 1);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}

void Calibrate() {
  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("Touch selected corners for calibration", 50, 100);
  TFT_Line(315, 1, 319, 1);
  TFT_Line(310, 10, 319, 1);
  TFT_Line(319, 5, 319, 1);
  TFT_Write_Text("first here", 250, 20);

  TP_TFT_Calibrate_Min();                      // Calibration of bottom left corner
  Delay_ms(500);

  TFT_Set_Pen(CL_BLACK, 3);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Line(315, 1, 319, 1);
  TFT_Line(310, 10, 319, 1);
  TFT_Line(319, 5, 319, 1);
  TFT_Write_Text("first here", 250, 20);

  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Line(0, 239, 0, 235);
  TFT_Line(0, 239, 5, 239);
  TFT_Line(0, 239, 10, 229);
  TFT_Write_Text("now here ", 15, 200);

  TP_TFT_Calibrate_Max();                      // Calibration of bottom left corner
  Delay_ms(500);
}


/////////////////////////
 TCScreen * const code CScreens[2] =
         {
                  &MainScreen           ,//   MainScreenScreenID = 32768
                  &ControlsScreen       //   ControlsScreenScreenID = 32769
         };

  const MainScreenScreenID = 32768;
  const ControlsScreenScreenID = 32769;

  unsigned int CurrentScreenID;

  TCScreen                MainScreen=
         {
         0x0000                ,//  MainScreen.Color
         320                   ,//  MainScreen.Width                     
         240                   ,//  MainScreen.Height                    
         3                     ,//  MainScreen.ObjectsCount             
         1                     ,//  MainScreen.CLabelsCount              
         Screen1_CLabels       ,//  MainScreen.CLabels
         0                     ,//  MainScreen.ImagesCount              
         0                     ,//  MainScreen.Images
         1                     ,//  MainScreen.CBoxesCount              
         Screen1_CBoxes        ,//  MainScreen.CBoxes
         1                     ,//  MainScreen.ProgressBarsCount         
         Screen1_ProgressBars  //  MainScreen._ProgressBars
         };

  TCBox                Box1 = 
         {
         32768                 , //   Box1.OwnerScreenID
         0                     , //   Box1.Order           
         0                     , //   Box1.Left           
         0                     , //   Box1.Top             
         320                   , //   Box1.Width           
         240                   , //   Box1.Height          
         1                     , //   Box1.Pen_Width       
         0x0000                , //   Box1.Pen_Color       
         1                     , //   Box1.Visible         
         1                     , //   Box1.Active          
         1                     , //   Box1.Transparent     
         1                     , //   Box1.Gradient        
         0                     , //   Box1.Gradient_Orientation    
         0x07FF                , //   Box1.Gradient_Start_Color    
         0x0000                , //   Box1.Gradient_End_Color      
         0xC618                , //   Box1.Color           
         0                     , //   Box1.PressColEnabled 
         0x8410                , //   Box1.Press_Color     
         0                     ,//  Box1.OnUpPtr
         0                     ,//  Box1.OnDownPtr
         0                     ,//  Box1.OnClickPtr
         0                      //  Box1.OnPressPtr
         };
  TProgressBar                 MainPageProgress;
char MainPageProgress_Caption[8] = "0\%";

  TCLabel          LoadingLabel = 
         {
         32768                 , //   LoadingLabel.OwnerScreenID
         2                     , //   LoadingLabel.Order          
         80                    , //   LoadingLabel.Left           
         80                    , //   LoadingLabel.Top             
         160                   , //   LoadingLabel.Width         
         32                    , //   LoadingLabel.Height       
         &LoadingLabel_Caption , //   LoadingLabel.Caption        
         Tahoma23x29_Regular   , //   LoadingLabel.FontName       
         0x0000                , //   LoadingLabel.Font_Color     
         1                     , //   LoadingLabel.Visible        
         1                     , //   LoadingLabel.Active        
         0                     ,//  LoadingLabel.OnUpPtr
         0                     ,//  LoadingLabel.OnDownPtr
         0                     ,//  LoadingLabel.OnClickPtr
         0                      //  LoadingLabel.OnPressPtr
         };
const char LoadingLabel_Caption[18] = "Loading..........";

  TCLabel                 * const code Screen1_CLabels[1]=
         {
         &LoadingLabel         
         };
  TCBox                   * const code Screen1_CBoxes[1]=
         {
         &Box1                 
         };
  TProgressBar               * const code Screen1_ProgressBars[1]=
         {
         &MainPageProgress     
         };


  TCScreen                ControlsScreen=
         {
         0x0000                ,//  ControlsScreen.Color
         320                   ,//  ControlsScreen.Width                     
         240                   ,//  ControlsScreen.Height                    
         10                    ,//  ControlsScreen.ObjectsCount             
         0                     ,//  ControlsScreen.CLabelsCount              
         0                     ,//  ControlsScreen.CLabels
         8                     ,//  ControlsScreen.ImagesCount              
         Screen2_Images        ,//  ControlsScreen.Images
         1                     ,//  ControlsScreen.CBoxesCount              
         Screen2_CBoxes        ,//  ControlsScreen.CBoxes
         1                     ,//  ControlsScreen.ProgressBarsCount         
         Screen2_ProgressBars  //  ControlsScreen._ProgressBars
         };

  TCBox                Box2 = 
         {
         32769                 , //   Box2.OwnerScreenID
         0                     , //   Box2.Order           
         0                     , //   Box2.Left           
         0                     , //   Box2.Top             
         320                   , //   Box2.Width           
         240                   , //   Box2.Height          
         1                     , //   Box2.Pen_Width       
         0x0000                , //   Box2.Pen_Color       
         1                     , //   Box2.Visible         
         1                     , //   Box2.Active          
         1                     , //   Box2.Transparent     
         0                     , //   Box2.Gradient        
         0                     , //   Box2.Gradient_Orientation    
         0x07FF                , //   Box2.Gradient_Start_Color    
         0x0000                , //   Box2.Gradient_End_Color      
         0x0000                , //   Box2.Color           
         0                     , //   Box2.PressColEnabled 
         0x8410                , //   Box2.Press_Color     
         0                     ,//  Box2.OnUpPtr
         0                     ,//  Box2.OnDownPtr
         0                     ,//  Box2.OnClickPtr
         0                      //  Box2.OnPressPtr
         };
  TImage               Led1;
  TImage               Led4;
  TImage               Switch3;
  TImage               Switch2;
  TImage               Led3;
  TImage               Switch1;
  TImage               Switch4;
  TImage               Led2;
  TProgressBar                 ProgressBar1;
char ProgressBar1_Caption[8] = "0\%";

  TImage                 * const code Screen2_Images[8]=
         {
         &Led1,                
         &Led4,                
         &Switch3,             
         &Switch2,             
         &Led3,                
         &Switch1,             
         &Switch4,             
         &Led2                 
         };
  TCBox                   * const code Screen2_CBoxes[1]=
         {
         &Box2                 
         };
  TProgressBar               * const code Screen2_ProgressBars[1]=
         {
         &ProgressBar1         
         };


  TMyScreen        CurrentMyScreen;


static void InitializeObjects() {



  MainPageProgress.OwnerScreenID   = 32768;
  MainPageProgress.Order           = 1;
  MainPageProgress.Left            = 60;
  MainPageProgress.Top             = 44;
  MainPageProgress.Width           = 200;
  MainPageProgress.Height          = 30;
  MainPageProgress.Pen_Width       = 1;
  MainPageProgress.Pen_Color       = 0x0000;
  MainPageProgress.Visible         = 1;
  MainPageProgress.Active          = 1;
  MainPageProgress.Transparent     = 1;
  MainPageProgress.Caption         = MainPageProgress_Caption;
  MainPageProgress.FontName        = Tahoma11x13_Regular;
  MainPageProgress.Font_Color      = 0x0000;
  MainPageProgress.Gradient        = 1;
  MainPageProgress.Gradient_Orientation    = 0;
  MainPageProgress.Gradient_Start_Color    = 0x07FF;
  MainPageProgress.Gradient_End_Color      = 0x07E0;
  MainPageProgress.Color           = 0xA65E;
  MainPageProgress.Background_Color = 0xFFFF;
  MainPageProgress.Min             = 0;
  MainPageProgress.Max             = 100;
  MainPageProgress.Position        = 0;
  MainPageProgress.Show_Position   = 1;
  MainPageProgress.Show_Percentage = 1;
  MainPageProgress.Smooth          = 1;
  MainPageProgress.Rounded         = 0;
  MainPageProgress.Corner_Radius      = 1;

  Led1.OwnerScreenID   = 32769;
  Led1.Order          = 1;
  Led1.Left           = 14;
  Led1.Top            = 20;
  Led1.Width          = 60;
  Led1.Height         = 64;
  Led1.Picture_Type        = 1;
  Led1.Picture_Ratio  = 1;
  Led1.Picture_Name   = Led1Off_jpg;
  Led1.Visible        = 1;
  Led1.Active         = 1;
  Led1.OnUpPtr         = 0;
  Led1.OnDownPtr       = 0;
  Led1.OnClickPtr      = Led1OnClick;
  Led1.OnPressPtr      = 0;

  Led4.OwnerScreenID   = 32769;
  Led4.Order          = 2;
  Led4.Left           = 245;
  Led4.Top            = 19;
  Led4.Width          = 60;
  Led4.Height         = 64;
  Led4.Picture_Type        = 1;
  Led4.Picture_Ratio  = 1;
  Led4.Picture_Name   = Led1Off_jpg;
  Led4.Visible        = 1;
  Led4.Active         = 1;
  Led4.OnUpPtr         = 0;
  Led4.OnDownPtr       = 0;
  Led4.OnClickPtr      = Led4OnClick;
  Led4.OnPressPtr      = 0;

  Switch3.OwnerScreenID   = 32769;
  Switch3.Order          = 3;
  Switch3.Left           = 168;
  Switch3.Top            = 132;
  Switch3.Width          = 60;
  Switch3.Height         = 64;
  Switch3.Picture_Type        = 1;
  Switch3.Picture_Ratio  = 1;
  Switch3.Picture_Name   = Switch3Off_jpg;
  Switch3.Visible        = 1;
  Switch3.Active         = 1;
  Switch3.OnUpPtr         = 0;
  Switch3.OnDownPtr       = 0;
  Switch3.OnClickPtr      = Switch3OnClick;
  Switch3.OnPressPtr      = 0;

  Switch2.OwnerScreenID   = 32769;
  Switch2.Order          = 4;
  Switch2.Left           = 91;
  Switch2.Top            = 132;
  Switch2.Width          = 60;
  Switch2.Height         = 64;
  Switch2.Picture_Type        = 1;
  Switch2.Picture_Ratio  = 1;
  Switch2.Picture_Name   = Switch2Off_jpg;
  Switch2.Visible        = 1;
  Switch2.Active         = 1;
  Switch2.OnUpPtr         = 0;
  Switch2.OnDownPtr       = 0;
  Switch2.OnClickPtr      = Switch2OnClick;
  Switch2.OnPressPtr      = 0;

  Led3.OwnerScreenID   = 32769;
  Led3.Order          = 5;
  Led3.Left           = 168;
  Led3.Top            = 20;
  Led3.Width          = 60;
  Led3.Height         = 64;
  Led3.Picture_Type        = 1;
  Led3.Picture_Ratio  = 1;
  Led3.Picture_Name   = Led1Off_jpg;
  Led3.Visible        = 1;
  Led3.Active         = 1;
  Led3.OnUpPtr         = 0;
  Led3.OnDownPtr       = 0;
  Led3.OnClickPtr      = Led3OnClick;
  Led3.OnPressPtr      = 0;

  Switch1.OwnerScreenID   = 32769;
  Switch1.Order          = 6;
  Switch1.Left           = 14;
  Switch1.Top            = 132;
  Switch1.Width          = 60;
  Switch1.Height         = 64;
  Switch1.Picture_Type        = 1;
  Switch1.Picture_Ratio  = 1;
  Switch1.Picture_Name   = Switch1Off_jpg;
  Switch1.Visible        = 1;
  Switch1.Active         = 1;
  Switch1.OnUpPtr         = 0;
  Switch1.OnDownPtr       = 0;
  Switch1.OnClickPtr      = Switch1OnClick;
  Switch1.OnPressPtr      = 0;

  Switch4.OwnerScreenID   = 32769;
  Switch4.Order          = 7;
  Switch4.Left           = 245;
  Switch4.Top            = 132;
  Switch4.Width          = 60;
  Switch4.Height         = 64;
  Switch4.Picture_Type        = 1;
  Switch4.Picture_Ratio  = 1;
  Switch4.Picture_Name   = Switch7Off_jpg;
  Switch4.Visible        = 1;
  Switch4.Active         = 1;
  Switch4.OnUpPtr         = 0;
  Switch4.OnDownPtr       = 0;
  Switch4.OnClickPtr      = Switch4OnClick;
  Switch4.OnPressPtr      = 0;

  Led2.OwnerScreenID   = 32769;
  Led2.Order          = 8;
  Led2.Left           = 91;
  Led2.Top            = 20;
  Led2.Width          = 60;
  Led2.Height         = 64;
  Led2.Picture_Type        = 1;
  Led2.Picture_Ratio  = 1;
  Led2.Picture_Name   = Led1Off_jpg;
  Led2.Visible        = 1;
  Led2.Active         = 1;
  Led2.OnUpPtr         = 0;
  Led2.OnDownPtr       = 0;
  Led2.OnClickPtr      = Led2OnClick;
  Led2.OnPressPtr      = 0;

  ProgressBar1.OwnerScreenID   = 32769;
  ProgressBar1.Order           = 9;
  ProgressBar1.Left            = 110;
  ProgressBar1.Top             = 213;
  ProgressBar1.Width           = 100;
  ProgressBar1.Height          = 17;
  ProgressBar1.Pen_Width       = 1;
  ProgressBar1.Pen_Color       = 0x0000;
  ProgressBar1.Visible         = 1;
  ProgressBar1.Active          = 1;
  ProgressBar1.Transparent     = 1;
  ProgressBar1.Caption         = ProgressBar1_Caption;
  ProgressBar1.FontName        = Tahoma11x13_Regular;
  ProgressBar1.Font_Color      = 0x0000;
  ProgressBar1.Gradient        = 1;
  ProgressBar1.Gradient_Orientation    = 0;
  ProgressBar1.Gradient_Start_Color    = 0x07FF;
  ProgressBar1.Gradient_End_Color      = 0x07E0;
  ProgressBar1.Color           = 0xA65E;
  ProgressBar1.Background_Color = 0xFFFF;
  ProgressBar1.Min             = 0;
  ProgressBar1.Max             = 100;
  ProgressBar1.Position        = 0;
  ProgressBar1.Show_Position   = 1;
  ProgressBar1.Show_Percentage = 1;
  ProgressBar1.Smooth          = 1;
  ProgressBar1.Rounded         = 0;
  ProgressBar1.Corner_Radius      = 1;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetCLabel(index)              CurrentMyScreen.CLabels[index]
#define GetImage(index)               CurrentMyScreen.Images[index]
#define GetCBox(index)                CurrentMyScreen.CBoxes[index]
#define GetProgressBar(index)           CurrentMyScreen.ProgressBars[index]

 void DeleteTrailingSpaces(char* str){
   char i;
   i = 0;
   while(1) {
   if(str[0] == ' ') {
      for(i = 0; i < strlen(str); i++) {
       str[i] = str[i+1];
      }
   }
   else
     break;
  }
 }

void DrawCLabel(TCLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Ext_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Const_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawImage(TImage *AImage) {
  if (AImage->Visible) {
    TFT_Ext_Image(AImage->Left, AImage->Top, AImage->Picture_Name, AImage->Picture_Ratio);
  }
}

void DrawCBox(TCBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void UpdatePBPosition(TProgressBar *AProgressBar) {
  int move_offset;
  unsigned int locPosition = 0;
  unsigned cnt = 0;
  char tmpStr[8];
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  move_offset = 10;
  locPosition = (float)(AProgressBar->Position - AProgressBar->Min) / (float)(AProgressBar->Max - AProgressBar->Min) * 100;
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Background_Color, AProgressBar->Pen_Width);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    if (caption_height >  AProgressBar->Height)
        caption_height =  AProgressBar->Height;
    if (caption_length >  AProgressBar->Width)
        caption_length =  AProgressBar->Width;
    if (AProgressBar->Position < (AProgressBar->Left + (AProgressBar->Width - caption_length) / 2)) {
    TFT_Rectangle(AProgressBar->Left + (AProgressBar->Width - caption_length) / 2,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2,
                  AProgressBar->Left + (AProgressBar->Width - caption_length) / 2 + caption_length,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2 + caption_height);
    }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius  + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100 , AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      if (locPosition > 0) {
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
        while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
              if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
              else
          TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
          cnt = cnt + move_offset + 3;
        }
      }
    // Clear caption frist
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          IntToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
    }
    else {
         IntToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
    }
    TFT_Set_Ext_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
    }
 }
}

void DrawProgressBar(TProgressBar *AProgressBar) {
  int move_offset = 0;
  unsigned int locPosition = 0;
  unsigned cnt = 0;
  char tmpStr[8];
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  move_offset = 10;
  locPosition = (float)(AProgressBar->Position - AProgressBar->Min) / (float)(AProgressBar->Max - AProgressBar->Min) * 100;
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      // First draw back rect
      TFT_Rectangle_Round_Edges(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height, AProgressBar->Corner_Radius);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius   + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      // First draw back rect
      TFT_Rectangle(AProgressBar->Left, AProgressBar->Top, AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
     while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
            if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
            else
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
        cnt = cnt + move_offset + 3;
     }
    }
    // Clear caption frist
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          IntToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
       }
       else {
         IntToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
       }
       TFT_Set_Ext_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
       TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
       TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
     }
 }
}

void SetCurrentMyScreenByIndex(unsigned aScreenID){
  memset(&CurrentMyScreen, 0, sizeof(CurrentMyScreen));
  if (aScreenID & 0x8000){
    aScreenID ^= 0x8000;
    CurrentMyScreen.Color        =  CScreens[aScreenID]->Color;
    CurrentMyScreen.Width        =  CScreens[aScreenID]->Width;
    CurrentMyScreen.Height       =  CScreens[aScreenID]->Height;
    CurrentMyScreen.ObjectsCount =  CScreens[aScreenID]->ObjectsCount;
    CurrentMyScreen.CLabelsCount   =  CScreens[aScreenID]->CLabelsCount;
    CurrentMyScreen.CLabels        =  CScreens[aScreenID]->CLabels;
    CurrentMyScreen.ImagesCount   =  CScreens[aScreenID]->ImagesCount;
    CurrentMyScreen.Images        =  CScreens[aScreenID]->Images;
    CurrentMyScreen.CBoxesCount   =  CScreens[aScreenID]->CBoxesCount;
    CurrentMyScreen.CBoxes        =  CScreens[aScreenID]->CBoxes;
    CurrentMyScreen.ProgressBarsCount   =  CScreens[aScreenID]->ProgressBarsCount;
    CurrentMyScreen.ProgressBars        =  CScreens[aScreenID]->ProgressBars;
  }
}

void DrawScreen(unsigned int aScreenID) {
 int order;
  unsigned short clabel_idx;
  TCLabel *local_clabel;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short cbox_idx;
  TCBox *local_cbox;
  unsigned short progress_bar_idx;
  TProgressBar *local_progress_bar;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  clabel_idx = 0;
  image_idx = 0;
  cbox_idx = 0;
  progress_bar_idx = 0;
  SetCurrentMyScreenByIndex(aScreenID);
  CurrentScreenID = aScreenID;

  if ((display_width != CurrentMyScreen.Width) || (display_height != CurrentMyScreen.Height)) {
    save_bled = TFT_BLED;
    save_bled_direction = TFT_BLED_Direction;
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(CurrentMyScreen.Width, CurrentMyScreen.Height);
    TFT_Set_Ext_Buffer(TFT_Get_Data);
    TP_TFT_Init(CurrentMyScreen.Width, CurrentMyScreen.Height, 0, 1);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentMyScreen.Color);
    display_width = CurrentMyScreen.Width;
    display_height = CurrentMyScreen.Height;
    TFT_BLED           = save_bled;
    TFT_BLED_Direction = save_bled_direction;
  }
  else
    TFT_Fill_Screen(CurrentMyScreen.Color);


  while (order < CurrentMyScreen.ObjectsCount) {
    if (clabel_idx < CurrentMyScreen.CLabelsCount) {
      local_clabel = GetCLabel(clabel_idx);
      if (order == local_clabel->Order) {
        clabel_idx++;
        order++;
        DrawCLabel(local_clabel);
      }
    }

    if (cbox_idx < CurrentMyScreen.CBoxesCount) {
      local_cbox = GetCBox(cbox_idx);
      if (order == local_cbox->Order) {
        cbox_idx++;
        order++;
        DrawCBox(local_cbox);
      }
    }

    if (image_idx  < CurrentMyScreen.ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

    if (progress_bar_idx  < CurrentMyScreen.ProgressBarsCount) {
      local_progress_bar = GetProgressBar(progress_bar_idx);
      if (order == local_progress_bar->Order) {
        progress_bar_idx++;
        order++;
        DrawProgressBar(local_progress_bar);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  clabel_order        = -1;
  image_order         = -1;
  cbox_order          = -1;
  progress_bar_order    = -1;
  //  CLabels
  for ( _object_count = 0 ; _object_count < CurrentMyScreen.CLabelsCount ; _object_count++ ) {
    local_clabel = GetCLabel(_object_count);
    if (local_clabel->Active == 1) {
      if (IsInsideObject(X, Y, local_clabel->Left, local_clabel->Top,
                         local_clabel->Width, local_clabel->Height) == 1) {
        clabel_order = local_clabel->Order;
        exec_clabel = local_clabel;
      }
    }
  }

  //  Images
  for ( _object_count = 0 ; _object_count < CurrentMyScreen.ImagesCount ; _object_count++ ) {
    local_image = GetImage(_object_count);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  //  CBoxes
  for ( _object_count = 0 ; _object_count < CurrentMyScreen.CBoxesCount ; _object_count++ ) {
    local_cbox = GetCBox(_object_count);
    if (local_cbox->Active == 1) {
      if (IsInsideObject(X, Y, local_cbox->Left, local_cbox->Top,
                         local_cbox->Width, local_cbox->Height) == 1) {
        cbox_order = local_cbox->Order;
        exec_cbox = local_cbox;
      }
    }
  }

  // ProgressBar
  for ( _object_count = 0 ; _object_count < CurrentMyScreen.ProgressBarsCount ; _object_count++ ) {
    local_progress_bar = GetProgressBar(_object_count);
    if (local_progress_bar->Active == 1) {
      if (IsInsideObject(X, Y, local_progress_bar->Left, local_progress_bar->Top,
                         local_progress_bar->Width, local_progress_bar->Height) == 1) {
        progress_bar_order = local_progress_bar->Order;
        exec_progress_bar = local_progress_bar;
      }
    }
  }

  _object_count = -1;
  if (clabel_order >  _object_count )
    _object_count = clabel_order;
  if (image_order >  _object_count )
    _object_count = image_order;
  if (cbox_order >  _object_count )
    _object_count = cbox_order;
  if (progress_bar_order >  _object_count )
    _object_count = progress_bar_order;
}


static void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_clabel         = 0;
  exec_image          = 0;
  exec_cbox           = 0;
  exec_progress_bar     = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        if (exec_clabel->OnPressPtr != 0) {
          exec_clabel->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == cbox_order) {
      if (exec_cbox->Active == 1) {
        if (exec_cbox->OnPressPtr != 0) {
          exec_cbox->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Box
    case 14: {
      if (PressedObject != 0) {
        exec_cbox = (TCBox*)PressedObject;
        if ((exec_cbox->PressColEnabled == 1) && (exec_cbox->OwnerScreenID == CurrentScreenID)) {
          DrawCBox(exec_cbox);
        }
        break;
      }
      break;
    }
  }

  exec_clabel          = 0;
  exec_image          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // CLabels
    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        if (exec_clabel->OnUpPtr != 0)
          exec_clabel->OnUpPtr();
        if (PressedObject == (TPointer)exec_clabel)
          if (exec_clabel->OnClickPtr != 0)
            exec_clabel->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Images
    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (TPointer)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // CBoxes
    if (_object_count == cbox_order) {
      if (exec_cbox->Active == 1) {
        if (exec_cbox->OnUpPtr != 0)
          exec_cbox->OnUpPtr();
        if (PressedObject == (TPointer)exec_cbox)
          if (exec_cbox->OnClickPtr != 0)
            exec_cbox->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_clabel         = 0;
  exec_image          = 0;
  exec_cbox           = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        PressedObject = (TPointer)exec_clabel;
        PressedObjectType = 10;
        if (exec_clabel->OnDownPtr != 0) {
          exec_clabel->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (TPointer)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == cbox_order) {
      if (exec_cbox->Active == 1) {
        if (exec_cbox->PressColEnabled == 1) {
          object_pressed = 1;
          DrawCBox(exec_cbox);
        }
        PressedObject = (TPointer)exec_cbox;
        PressedObjectType = 14;
        if (exec_cbox->OnDownPtr != 0) {
          exec_cbox->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  TP_TFT_Rotate_180(0);
}

void Init_Ext_Mem() {
  // Initialize SPI
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
  Delay_ms(10);

  // Initialize MMC
  if (!Mmc_Fat_Init()) {
    // Reinitialize SPI at higher speed
    SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV4, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);

    // Open resource file for read
    Mmc_Fat_Assign("Source_r.RES", 0);
    Mmc_Fat_Reset(&res_file_size);
  }
}

void Start_TP() {
  Init_MCU();

  Init_Ext_Mem();

  InitializeTouchPanel();

  Delay_ms(1000);
  TFT_Fill_Screen(0);
  Calibrate();
  TFT_Fill_Screen(0);

  InitializeObjects();
  display_width = MainScreen.Width;
  display_height = MainScreen.Height;
  DrawScreen(32768);
}
